;**********************************************************************
;                    PIC12F1501
;                   -----------U-----------  
;             +5V --|Vdd               Vss|-- GND
; zen nabjen --|GP5/oscin     GP0/PGD|-- vstup buzzer (vibrace)
; propojky (mode) --|GP4/oscout    GP1/PGC|-- vstup piezo (NCO), buzzer2
; propojky (test) --|GP3/MCLR/Vpp  GP2/INT|-- propojky (mode)
;                   -----------------------
; fosc = 31 kHz, 1 s = 7750 tcy, 1 tcy ~ 129 s
;**********************************************************************

	list    p=12f1501			; list directive to define processor
	#include <p12F1501.inc>		; processor specific variable definitions
  	errorlevel  -302			; suppress message 302 from list file

	__CONFIG	_CONFIG1, _FOSC_INTOSC & _WDTE_ON & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
	__CONFIG	_CONFIG2, _WRT_OFF & _STVREN_OFF & _BORV_HI & _LPBOR_OFF & _LVP_OFF

tmp 		equ	0x70	;cykly
tmp1		equ	0x71	;zpoovac smyky
tmp2		equ	0x72	;zpoovac smyky
randh		equ	0x74	;genertor pseudonhodnho sla
randl		equ	0x75	;genertor pseudonhodnho sla
ncoltmp		equ	0x78	;pomocn registr pi zmn kmitotu NCO
;ncohtmp		equ	0x79	;pomocn registr pi zmn kmitotu NCO

#define	BUZZ	PORTA,0		; vstup na buzzer, zapne v 1, vypne v 0
#define	BUZZ2	PORTA,1		; vstup na buzzer2, zapne v 1, vypne v 0
#define	CHRG	PORTA,5		; vstup pro zen nabjen, nabj v 0, nenabj v 1

	ORG 0x00
;	goto start

;	ORG 0x04
;	retfie

;	ORG 0x0F
;start
	BANKSEL	OSCCON
	movlw	b'00000010'
	movwf	OSCCON		; 31 kHz, Internal oscillator block

	BANKSEL	PORTA		;(0)
	clrf	PORTA		;(0) Init PORTA
	movlw	0xa1
	movwf	randl		; inicializace promnn genertoru pseudonhodnch sel
	BANKSEL	LATA		;(2) Data Latch
	clrf	LATA
	BANKSEL	ANSELA		;(3)
	clrf	ANSELA		;digital I/O
	BANKSEL	TRISA		;(1)
	clrf	TRISA		;RA0-RA2, RA4, RA5 all outputs
	bcf 	OPTION_REG,7;(1) enable WPU
	movlw	b'00011100'	; 32 sec.
	movwf	WDTCON		; Watch-dog prescaler
	BANKSEL	WPUA		;(4)
	movlw	B'00010100'
	movwf	WPUA		;(4) Pull-up RA2,RA4

;	call	d020	;test dlky zpodn
;	goto	$-1		;test
batest	;zen nabjen a test malho napt
	clrwdt
;zapni Vref
	banksel FVRCON		;(2)
	movlw	b'10001000'	; voltage reference 2.048 V enable
	movwf	FVRCON		;(2)
;zapni DAC
	movlw	b'10000000'	; DAC enabled, ref=Vdd
	movwf	DACCON0		;(2)
	movlw	b'00010011'	; DAC stage = .19 pro 3,45 V
	movwf	DACCON1		;(2)
;zapni kompartor
;	banksel CM1CON0		;(2)
	movlw	b'10000000'	; comparator enabled to internal output, low power, no hysteresis
	movwf	CM1CON0		;(2)
	movlw	b'00010100'	; no interupt, (+)DACout, (-)FVR Voltage Reference,
	movwf	CM1CON1		;(2)
; Max. doba ustlen: kompartor 10 s, DAC 10 s, FRV 40 s.
; Pi taktu 31 kHz 1 tcy ~ 129 s nen teba na ustlen ekat.
	btfss	CM1CON0,C1OUT	;(2)test 3,45 V
	goto	lowbatt			; mal napt baterie, U < 3,45 V
;test
;	goto	krtek
;\test
	movlw	b'00010000'		; DAC stage = .16 (4,1 V)
	movwf	DACCON1			;(2)
	btfsc	CM1CON0,C1OUT	;(2) test 4,1 V
	goto	chrgstop		; ukoni nabjen, U > 4,1 V
; napt je v rozsahu 3,45 a 4,1 V
	call	batestoff	; vypne Vref, DAC a kompartor (nastav bank 0)
	bcf 	CHRG		;(0) zapni nabjen (0)
	goto	modesel		; vbr funkce podle propojek
chrgstop	;baterka nabit - vypni nabjen	
	call	batestoff	; vypne Vref, DAC a kompartor (nastav bank 0)
	bsf 	CHRG		;(0) vypni nabjen (1)
modesel		; vbr funkce podle propojek
; naten propojek
	BANKSEL	TRISA		;(1)
	movlw	B'11011100'
	movwf	TRISA			;(1) RA2-RA4 vstupy
	banksel PORTA			;(0)
	movf	PORTA,w			;(0) nati port
	movwf	tmp
	BANKSEL	TRISA			;(1)
	clrf	TRISA			;(1) RA0-RA2, RA4, RA5 vstupy
	banksel PORTA		;(0)
	rrf 	tmp,f		; posune stav propojek
	rrf 	tmp,w		; posune stav propojek
	andlw	0x07		; jen spodn 3 bity
	addwf	PCL,f
; propojky
	goto	krt2tst	; 000 propojka on  - on  - on (krtek2 test)
	goto	slimtst	; 001 propojka on  - off - on (slimak test)
	goto	krtek2	; 010 propojka on  - on  - off (krtek2 - 2x buzzer)
	goto	slimak	; 011 propojka on  - off - off (slimak)
	goto	kunatst	; 100 propojka off - on  - on (kuna test)
	goto	krttest	; 101 propojka off - off - on (krtek - test)
	goto	kuna	; 110 propojka off - on  - off (kuna)
	goto	krtek	; 111 propojka off - off - off (krtek - buzzer + piezo)

lowbatt	;mal napt baterie
	call	batestoff	; vypne Vref, DAC a kompartor (nastav bank 0)
	bcf 	CHRG		;(0) zapni nabjen (0)
	bcf 	BUZZ		;(0) vypni zt (0 - zatm LED)
	call	piezooff
	sleep
	goto	batest

krtek	;plac smyka na krtky
		; na vstupu "buzzer" se generuje signl pro spnn bzuku
		; na vstupu "piezo" se generuje plaic zvuk s promnnm kmitotem.
	btfss	randh,4
	bsf 	BUZZ		; zapni buzzer
	btfss	randh,1
	call	piezokrt	; zapni piezo
nobuzz
	call	random
	movf	randl,w
;	movlw	.20			; pevn interval pro test
	movwf	tmp
	call	d020
	decfsz	tmp,f		;tmp>=1,tmp(0)=255
	goto	$-2
	call	d500
	bcf 	BUZZ		; vypni buzzer
	bcf 	BUZZ2		; vypni buzzer2
	call	random
	movf	randl,w
;	movlw	.2			; zkrcen intervalu pro test
	movwf	tmp
	call	d125
	decfsz	tmp,f		;tmp>=1,tmp(0)=255
	goto	$-2

	goto	batest

krttest	; test reimu krtek - stdav aktivn vstup "buzzer" a "piezo"
		; na vstupu "piezo" se generuje plaic zvuk s promnnm kmitotem.
	call	piezooff
	bsf 	BUZZ		; zapni buzzer
	call	d500 		; ekej 500 ms
	call	d500 		; ekej
	bcf 	BUZZ		; vypni buzzer
	call	d125		; ekej 125 ms
	call	pikrttst	; zapni piezo 
	call	d125		; ekej 125 ms
	goto	batest

kuna	; aktivn jen vstup PIEZO
		; kuna 17 kHz/500 ms + nhodn pauza 58 a 1435 ms
		; 17 kHz - nastavit NCO 2227,--> nco-H = 8, nco-L = 179
	call	piezooff
	call	random
	movf	randl,w
	movwf	tmp
	call	d005
	decfsz	tmp,f		;tmp>=1,tmp(0)=255
	goto	$-2
	call	d050
kunatst	; trval tn (17 kHz)
	movlw	.179	; nco-L = 179
	movwf	ncoltmp
	movlw	.1  	; nco-H = 8
	call	piezoset
	call	d500
	goto	batest

slimak	; aktivn jen vstup PIEZO
		; slimci 9 kHz/125 ms + pauza 1 s
		; 9 kHz - nastavit NCO 1179,--> nco-H = 4, nco-L = 155
	call	piezooff
	call	d500
	call	d500
slimtst	; trval tn (9 kHz)
	movlw	.155	; nco-L
	movwf	ncoltmp
	movlw	.2  	; nco-H
	call	piezoset
	call	d125
	goto	batest

krtek2	;plac smyka na krtky - 2x buzzer
		; na vstupu "buzzer" i "piezo" se generuje signl pro spnn bzuku
	call	piezooff
	movf	randh,w
	andlw	0x03		; jen spodn 2 bity
	addwf	PCL,f
; pseudonhodn vbr buzzer1 a buzzer2
	goto	nobuzz	; 00 nic
	goto	buzzer2	; 01 zapni buzzer2
	goto	nobuzz	; 10 nic
;	goto	buzzer1	; 11 zapni buzzer1
buzzer1
	bsf 	BUZZ		; zapni buzzer1
	goto	nobuzz
buzzer2
	bsf 	BUZZ2		; zapni buzzer2
	goto	nobuzz

krt2tst	; test reimu krtek - stdav aktivn vstup "buzzer" a "piezo"
		; vstupem "piezo" se spn druh bzuk.
	call	piezooff
	bsf 	BUZZ		; zapni buzzer
	call	d500 		; ekej 500 ms
	call	d500 		; ekej
	bcf 	BUZZ		; vypni buzzer
	call	d125		; ekej 125 ms

	bsf 	BUZZ2		; zapni buzzer2
	call	d500 		; ekej 500 ms
	call	d500 		; ekej
	bcf 	BUZZ2		; vypni buzzer2

	call	d125		; ekej 125 ms
	goto	batest

;;;;;;;;;;;;;;;;
; podprogramy

d500 ; zpodn 499,23 ms (3869 tcy pro 31 kHz)(.90 a .13 bez clrwdt)(69+13~499,226)
;	BANKSEL	0
	movlw	.69
	movwf	tmp2
	movlw	.13
	movwf	tmp1
	clrwdt
	decfsz	tmp1,f
	goto	$-2
	decfsz	tmp2,f
	goto	$-6
	return

d125 ; smyka 124,5 ms pro tmp2=240 (132,8 ms pro tmp2=0)
	movlw	.240
	goto	dset

d050 ; zpodn 50 ms (49,7 ms, 385 tcy pro 31 kHz)
	movlw	.95
	goto	dset

d020 ; zpodn 20 ms (155 tcy)
	movlw	.37
	goto	dset

;d015 ; zpodn asi 14,84 ms (115 tcy)
;	movlw	.27
;	goto	dset

;d010 ; zpodn 9,68 ms
;	movlw	.17
;	goto	dset

;d001 ; zpodn 1,16 ms
;	movlw	.1
;	goto	dset

d005 ; zpodn 5,03 ms
	movlw	.8
dset
	movwf	tmp2
	clrwdt
	decfsz	tmp2,f
	goto	$-2
	return
 
;piezo1k
;	movlw	.131	; nco-L
;	movwf	ncoltmp
;	movlw	.0  	; nco-H
piezoset
	banksel	NCO1CON		; movlb	.9
	movwf	NCO1INCH	;(9)
	movlw	b'11000000'	; NCO enabled,50% duty
	movwf	NCO1CON		;(9)
	movlw	b'00000000'	; HFINTOSC
	movwf	NCO1CLK		;(9)
	movf	ncoltmp,w
	movwf	NCO1INCL	;(9)
	banksel PORTA
	return

piezokrt
	btfss	randh,4
	return				; zapnut buzzer - nedlej nic
pikrttst
	banksel	NCO1CON		; movlb	.9
	movlw	b'11000000'	; NCO enabled,50% duty
	movwf	NCO1CON		;(9)
	movlw	b'00000000'	; HFINTOSC
	movwf	NCO1CLK		;(9)
	movlw	0x00
	movwf	NCO1INCH	;(9)
	movlw	.196		; 131 = 1 kHz
	movwf	NCO1INCL	;(9)
	movwf	ncoltmp
	clrwdt
	incf	ncoltmp,w	; zvti kmitoet
	btfsc	STATUS,Z
	incf	NCO1INCH,f	;(9) horn byte +1, kdy "0" (incf nedl carry)
	btfss	NCO1INCH,1
	goto	$-7
	decf	NCO1INCH,f
	decf	ncoltmp,w	; zmeni kmitoet
	call	d125

	movwf	NCO1INCL	;(9)
	movwf	ncoltmp
	clrwdt
	decf	ncoltmp,w	; zmeni kmitoet
	btfsc	STATUS,Z
	decf	NCO1INCH,f	;(9) horn byte -1, kdy "0" (incf nedl carry)
	btfsc	NCO1INCH,0
	goto	$-7

	call	piezooff

	banksel PORTA
	return

piezooff
	banksel	NCO1CON		; movlb	.9
	clrf	NCO1CON		;(9) NCO disabled
	banksel PORTA
	bcf 	PORTA,1		;(0) log.0 na RA1
	return

;16-bit random generator (Mark Jeronimus of Digital Mosular)
;The 0xA1 is part of a prime polynom value 0xA1A1 which generates a full LFSR
;http://www.piclist.com/techref/microchip/rand8bit.htm
random:
	bcf 	STATUS,C
	rrf 	randh,f
	rrf 	randl,f
	btfss	STATUS,C
	return
	movlw	0xa1
	xorwf	randh,f
	xorwf	randl,f
	return

batestoff	; vypne napovou referenci, DAC a kompartor pro zmenen spoteby
	banksel FVRCON		;(2)
	clrf	FVRCON		;(2)vypni Vref
	clrf	DACCON0		;(2)vypni DAC
	clrf	CM1CON0		;(2)vypni kompartor
	banksel PORTA		;(0)
	return

;	ORG 0x3f0
;	DE	"krtkovac 0.1a"

	END